/*
 * Copyright (C) 2019  CZ.NIC, z. s. p. o.
 *
 * This file is part of FRED.
 *
 * FRED is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * FRED is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with FRED.  If not, see <https://www.gnu.org/licenses/>.
 */

#ifndef STRONG_TYPE_COMPARISON_OPERATORS_HH_2E4D5C74E7E39CF477C1962932A44064//date "+%s.%N"|md5sum|tr "[a-f]" "[A-F]"
#define STRONG_TYPE_COMPARISON_OPERATORS_HH_2E4D5C74E7E39CF477C1962932A44064

#include "libpg/util/strong_type.hh"

#include <type_traits>

namespace LibPg {
namespace Util {

template <typename Type, typename Tag,
          std::enable_if_t<std::is_same<decltype(std::declval<Type>() == std::declval<Type>()), bool>::value>* = nullptr>
constexpr bool operator==(const StrongType<Type, Tag>& lhs, const StrongType<Type, Tag>& rhs)
{
    return lhs.get() == rhs.get();
}

template <typename Type, typename Tag,
          std::enable_if_t<std::is_same<decltype(std::declval<Type>() != std::declval<Type>()), bool>::value>* = nullptr>
constexpr bool operator!=(const StrongType<Type, Tag>& lhs, const StrongType<Type, Tag>& rhs)
{
    return lhs.get() != rhs.get();
}

template <typename Type, typename Tag,
          std::enable_if_t<std::is_same<decltype(std::declval<Type>() < std::declval<Type>()), bool>::value>* = nullptr>
constexpr bool operator<(const StrongType<Type, Tag>& lhs, const StrongType<Type, Tag>& rhs)
{
    return lhs.get() < rhs.get();
}

template <typename Type, typename Tag,
          std::enable_if_t<std::is_same<decltype(std::declval<Type>() <= std::declval<Type>()), bool>::value>* = nullptr>
constexpr bool operator<=(const StrongType<Type, Tag>& lhs, const StrongType<Type, Tag>& rhs)
{
    return lhs.get() <= rhs.get();
}

template <typename Type, typename Tag,
          std::enable_if_t<std::is_same<decltype(std::declval<Type>() >= std::declval<Type>()), bool>::value>* = nullptr>
constexpr bool operator>=(const StrongType<Type, Tag>& lhs, const StrongType<Type, Tag>& rhs)
{
    return lhs.get() >= rhs.get();
}

template <typename Type, typename Tag,
         std::enable_if_t<std::is_same<decltype(std::declval<Type>() > std::declval<Type>()), bool>::value>* = nullptr>
constexpr bool operator>(const StrongType<Type, Tag>& lhs, const StrongType<Type, Tag>& rhs)
{
    return lhs.get() > rhs.get();
}

}//namespace LibPg::Util
}//namespace LibPg

#endif//STRONG_TYPE_COMPARISON_OPERATORS_HH_2E4D5C74E7E39CF477C1962932A44064
