/*
 * Copyright (C) 2019  CZ.NIC, z. s. p. o.
 *
 * This file is part of FRED.
 *
 * FRED is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * FRED is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with FRED.  If not, see <https://www.gnu.org/licenses/>.
 */

#ifndef STRONG_TYPE_STREAM_OPERATORS_HH_EC7DE0D8EF23BBF394F1B917E96E486D//date "+%s.%N"|md5sum|tr "[a-f]" "[A-F]"
#define STRONG_TYPE_STREAM_OPERATORS_HH_EC7DE0D8EF23BBF394F1B917E96E486D

#include "libpg/util/strong_type.hh"

#include <iosfwd>

namespace LibPg {
namespace Util {

template <typename Type, typename Tag,
          std::enable_if_t<std::is_same<decltype(std::declval<std::ostream&>() << std::declval<Type>()), std::ostream&>::value>* = nullptr>
std::ostream& operator<<(std::ostream& out, const StrongType<Type, Tag>& value)
{
    return out << value.get();
}

template <typename Type, typename Tag,
          std::enable_if_t<std::is_same<decltype(std::declval<std::istream&>() >> std::declval<Type&>()), std::istream&>::value>* = nullptr>
std::istream& operator>>(std::istream& src, StrongType<Type, Tag>& dst)
{
    struct ReadFrom
    {
        void operator()(Type& dst)const { src >> dst; }
        std::istream& src;
    };
    dst.invoke(ReadFrom{src});
    return src;
}

}//namespace LibPg::Util
}//namespace LibPg

#endif//STRONG_TYPE_STREAM_OPERATORS_HH_EC7DE0D8EF23BBF394F1B917E96E486D
